package net.benwilcock.service.handlerchain;

import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPHeader;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ClientAuthenticationSOAPHandler extends
		BaseHandler<SOAPMessageContext> implements
		SOAPHandler<SOAPMessageContext> {
	private final String HANDLER_NAME = "ClientAuthenticationSOAPHandler";
	private static Log log = LogFactory.getLog(ClientAuthenticationSOAPHandler.class);

	public ClientAuthenticationSOAPHandler() {
		super();
		super.setHandlerName(HANDLER_NAME);
	}

	public boolean handleMessage(SOAPMessageContext smc) {
		log.trace("------------------------------------");
		log.trace("In SOAPHandler " + HandlerName + ":handleMessage()");

		boolean direction = ((Boolean) smc
				.get(SOAPMessageContext.MESSAGE_OUTBOUND_PROPERTY))
				.booleanValue();

		String USERID = (String) smc
				.get(AuthenticationHandlerConstants.REQUEST_USERID);
		String PASSWORD = (String) smc
				.get(AuthenticationHandlerConstants.REQUEST_PASSWORD);

		if (direction) {
			log.trace("direction = outbound");

			// ......
			try {

				/*
				 * add a header with the authentication info into the SOAP
				 * message:
				 * 
				 * <env:Header> <nsAuthen:authnHeader
				 * xmlns:nsAuthn="http://cardservice.handler.jaxws.company.com/authn">
				 * <nsAuthn:id>yyang</nsAuthn:id> <nsAuthn:password>mypassword</nsAuthn:password>
				 * </nsAuthn:authnHeader> </env:Header>
				 */

				// get SOAP envelope from SOAP message
				SOAPEnvelope envelope = smc.getMessage().getSOAPPart()
						.getEnvelope();

				// create instance of SOAP factory
				SOAPFactory soapFactory = SOAPFactory.newInstance();

				// create SOAP elements specifying prefix and URI
				SOAPElement headerElm = soapFactory.createElement(
						AuthenticationHandlerConstants.AUTHN_LNAME,
						AuthenticationHandlerConstants.AUTHN_PREFIX,
						AuthenticationHandlerConstants.AUTHN_URI);
				SOAPElement idElm = soapFactory.createElement("id",
						AuthenticationHandlerConstants.AUTHN_PREFIX,
						AuthenticationHandlerConstants.AUTHN_URI);

				// attach value to id element
				idElm.addTextNode(USERID);
				SOAPElement passwdElm = soapFactory.createElement("password",
						AuthenticationHandlerConstants.AUTHN_PREFIX,
						AuthenticationHandlerConstants.AUTHN_URI);

				// attach value to password element
				passwdElm.addTextNode(PASSWORD);

				// add child elements to the root element
				headerElm.addChildElement(idElm);
				headerElm.addChildElement(passwdElm);

				// create SOAPHeader instance for SOAP envelope
				SOAPHeader sh = envelope.addHeader();

				// add SOAP element for header to SOAP header object
				sh.addChildElement(headerElm);
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		} else {
			log.trace("direction = inbound");
		}
		// ......

		log.trace("Exiting SOAPHandler " + HandlerName + ":handleMessage()");
		log.trace("------------------------------------");

		return true;
	}

	public Set<QName> getHeaders() {
		return null;
	}
}
