package net.benwilcock.service.handlerchain;

import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class ServiceAuthenticationSOAPHandler extends
		BaseHandler<SOAPMessageContext> implements
		SOAPHandler<SOAPMessageContext> {
	private final String HANDLER_NAME = "ServiceAuthenticationSOAPHandler";
	private static Log log = LogFactory.getLog(ServiceAuthenticationSOAPHandler.class);

	public ServiceAuthenticationSOAPHandler() {
		super();
		super.setHandlerName(HANDLER_NAME);
	}

	public boolean handleMessage(SOAPMessageContext smc) {
		boolean exit = false;
		log.trace("------------------------------------");
		log.trace("In SOAPHandler " + HandlerName + ":handleMessage()");

		boolean direction = ((Boolean) smc
				.get(SOAPMessageContext.MESSAGE_OUTBOUND_PROPERTY))
				.booleanValue();
		if (direction) {
			log.trace("direction = outbound");
		} else {
			log.trace("direction = inbound");

			// ......
			try {
				// get SOAP envelope from SOAP message
				SOAPEnvelope se = smc.getMessage().getSOAPPart().getEnvelope();

				// get the headers from envelope
				SOAPHeader sh = se.getHeader();
				if (sh == null) {
					log.trace("--- No headers found in the input SOAP request");
					exit = false;
				} else {
					// call method to process header
					exit = processSOAPHeader(sh);
				}
			} catch (Exception ex) {
				ex.printStackTrace();
				exit = false;
			}
			// ......
		}

		log.trace("Exiting SOAPHandler " + HandlerName + ":handleMessage()");
		log.trace("------------------------------------");
		
		if (exit){
			smc.put(AuthenticationHandlerConstants.AUTHN_STAUTS, true);
		}else{
			smc.put(AuthenticationHandlerConstants.AUTHN_STAUTS, true);
		}		
		smc.setScope(AuthenticationHandlerConstants.AUTHN_STAUTS, MessageContext.Scope.APPLICATION);
		
		return exit;
	}

	private boolean processSOAPHeader(SOAPHeader sh) {
		boolean authenticated = false;

		java.util.Iterator childElems = sh.getChildElements(new QName(
				AuthenticationHandlerConstants.AUTHN_URI,
				AuthenticationHandlerConstants.AUTHN_LNAME));
		SOAPElement child;
		// iterate through child elements
		while (childElems.hasNext()) {
			child = (SOAPElement) childElems.next();
			authenticated = processSOAPHeaderInfo(child);
		}
		return authenticated;
	}

	private boolean processSOAPHeaderInfo(SOAPElement e) {

		SOAPElement child;
		QName name;
		String _id = null;
		String _password = null;

		// get an iterator on child elements of SOAP element
		java.util.Iterator childElems = e.getChildElements();

		// loop through child elements
		while (childElems.hasNext()) {
			// get next child element
			Object elem = childElems.next();
			if (elem instanceof SOAPElement) {

				child = (SOAPElement) elem;
				name = (QName) child.getElementQName();

				// get the value of id element
				if (name.getLocalPart().equals("id")) {
					log.trace("\t\tid =" + child.getValue());
					_id = child.getValue();
				}

				// get the value of password element
				if (name.getLocalPart().equals("password")) {
					log.trace("\t\tpassword =" + child.getValue());
					_password = child.getValue();
				}
			}
		}
		return authenticate(_id, _password);
	}

	/**
	 * In reality, we may login to a credential store.
	 */
	private boolean authenticate(String _id, String _password) {

		// client authentication information
		final String USERID = "yyang";
		final String PASSWORD = "mypassword";

		if (_id.equals(USERID) && _password.equals(PASSWORD)) {
			log.trace("Authenticated by " + HandlerName + "!");
			return true;
		} else {
			log.trace("Could not authenticate ,WRONG id or password");
		}
		return false;
	}

	public Set<QName> getHeaders() {
		return null;
	}

}
